import json


class BaseFormatter(object):
    def format(self, data):
        raise NotImplementedError()

    def _parse(self, data):
        alternatives = (obj['alternatives'][0]
                        for obj in data['results'])
        return ({
            'transcript': obj['transcript'],
            'confidence': obj['confidence']
        } for obj in alternatives)


class BaseJsonFormatter(BaseFormatter):
    def __init__(self, indent=2):
        self.indent = indent


class OriginalJsonFormatter(BaseJsonFormatter):
    def format(self, data):
        return json.dumps(data, indent=self.indent)


class SimpleJsonFormatter(BaseJsonFormatter):
    def format(self, data):
        objects = list(self._parse(data))
        return json.dumps(objects, indent=self.indent)


class HTMLFormatter(BaseFormatter):
    def format(self, data):
        results = (obj['transcript']
                   for obj in self._parse(data))
        lines = ("{spaces}<p>{line}</p>\n".format(
            spaces=(' ' * 4), line=line) for line in results)
        return "<html>\n{}</html>".format(''.join(lines))


class MarkdownFormatter(BaseFormatter):
    def format(self, data):
        results = (obj['transcript']
                   for obj in self._parse(data))
        lines = ("{line}\n\n".format(line=line) for line in results)

        return ''.join(lines).rstrip()

FORMATTERS = {
    'html': HTMLFormatter,
    'markdown': MarkdownFormatter,
    'simple': SimpleJsonFormatter,
    'original': OriginalJsonFormatter
}


def get_formatter(formatter_name):
    return FORMATTERS[formatter_name]



"""Main commands speech2text.

# $ speech2text -i input.mp3 -f html (default) -o output.html
# optional: content type, formatters?
"""

import click
import progressbar

from .config import SUPPORTED_AUDIO_MODELS, SUPPORTED_CONTENT_TYPES
from . import recognize_speech
from .formatters import get_formatter


@click.command()
@click.option('-i', '--input', type=click.Path(exists=True), required=True)
@click.option('-u', '--username', envvar='IBM_USERNAME', required=True)
@click.option('-p', '--password', prompt=True,
              hide_input=True, envvar='IBM_PASSWORD')
@click.option('-f', '--formatter',
              type=click.Choice(['html', 'markdown', 'json', 'original']),
              default='html')
@click.option('-c', '--content-type',
              type=click.Choice(SUPPORTED_CONTENT_TYPES), required=False)
@click.option('-m', '--audio-model',
              type=click.Choice(SUPPORTED_AUDIO_MODELS), required=False)
@click.option('--progress/--no-progress', default=True)
@click.argument('output', type=click.Path(exists=False), required=True)
def speech_to_text(input, username, password, formatter,
                   content_type, audio_model, progress, output):

    APP = {
        'progress_bar': None
    }

    def progress_callback(data, progress, total_size):
        if not data and APP['progress_bar'] is None:
            return

        if APP['progress_bar'] is None:
            click.echo("Starting Upload.")
            APP['progress_bar'] = progressbar.ProgressBar(
                maxval=total_size,
                widgets=[progressbar.Bar('=', '[', ']'), ' ',
                         progressbar.Percentage()])
            APP['progress_bar'].start()
        APP['progress_bar'].update(progress)
        if not data:
            APP['progress_bar'].finish()
            click.echo("Upload finished. Waiting for Transcript")

    _callback = progress_callback if progress else None

    result = recognize_speech(username, password, audio_file_path=input,
                              forced_mime_type=content_type,
                              audio_model=audio_model,
                              progress_callback=_callback)

    FormatterClass = get_formatter(formatter)
    formatted_output = FormatterClass().format(result)

    with open(output, 'w') as output_file:
        output_file.write(formatted_output)

    click.echo('Speech > Text finished.')



from mimetypes import MimeTypes

from .config import SUPPORTED_CONTENT_TYPES


def guess_mime_type(audio_file_path, forced_mime_type=None):
    mime = MimeTypes()
    mime_type = forced_mime_type or mime.guess_type(audio_file_path)[0]
    if mime_type not in SUPPORTED_CONTENT_TYPES:
        raise ValueError('MimeType {} not recognized or supported for '
                         'audio file {}'.format(mime_type, audio_file_path))
    return mime_type



import os
import ast

from setuptools import setup
from setuptools.command.test import test as TestCommand

VALUES = {
    '__version__': None,
    '__title__': None,
    '__description__': None
}

with open('speech_to_text/__init__.py', 'r') as f:
    tree = ast.parse(f.read())
    for node in tree.body:
        if node.__class__ != ast.Assign:
            continue
        target = node.targets[0]
        if target.id in VALUES:
            VALUES[target.id] = node.value.s

if not all(VALUES.values()):
    raise RuntimeError("Can't locate values to init setuptools hook.")


version = VALUES['__version__']
project_name = os.path.basename(os.path.dirname(os.path.realpath(__file__)))
package_name = 'speech_to_text'

project_url = 'http://github.com/rmotr/{project_name}'.format(
    project_name=project_name)


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass to py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = ["-s", "tests.py"]

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import sys
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)

setup(
    name=VALUES['__title__'],
    version=version,
    description=VALUES['__description__'],
    url=project_url,
    download_url="{url}/tarball/{version}".format(
        url=project_url, version=version),
    author='Santiago Basulto',
    author_email='santiago@rmotr.com',
    license='MIT',
    scripts=['main.py'],
    packages=[package_name],
    maintainer='Santiago Basulto',
    install_requires=[
        "progressbar",
        "click==6.7",
        "watson-developer-cloud==0.25.1"
    ],
    dependency_links=[
        "https://github.com/niltonvolpato/python-progressbar/tarball/master#egg=progressbar"
    ],
    tests_require=[
        "mock==2.0.0",
        "pytest==3.0.6"
    ],
    entry_points={
        'console_scripts': [
            'speech_to_text = main:speech_to_text'
        ]
    },
    zip_safe=True,
    cmdclass={'test': PyTest},
)



from . import formatters


SUPPORTED_CONTENT_TYPES = (
    'audio/L16',
    'audio/x-flac',
    'audio/x-wav',
    'audio/ogg',
    'audio/basic'
)

SUPPORTED_AUDIO_MODELS = (
    'ar-AR_BroadbandModel',
    'en-UK_BroadbandModel',
    'en-UK_NarrowbandModel',
    'en-US_BroadbandModel',
    'en-US_NarrowbandModel',
    'es-ES_BroadbandModel',
    'es-ES_NarrowbandModel',
    'fr-FR_BroadbandModel',
    'ja-JP_BroadbandModel',
    'ja-JP_NarrowbandModel',
    'pt-BR_BroadbandModel',
    'pt-BR_NarrowbandModel',
    'zh-CN_BroadbandModel',
    'zh-CN_NarrowbandModel'
)
